# Woclema - Weblog, Originally Command Line Editable/Manageable
# Copyright (C) 2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package Woclema::Category;

use strict;
use warnings FATAL => 'all';
use diagnostics;

use Woclema::DB;

my @id_to_name;	# Cache, [id] => "name"
my @id_to_desc;	# Cache, [id] => "description"


=item add
Add a category.
Does not use transactions.

0: name
(1: description)
r: category ID
=cut
sub add_unsafe($$) {
	my ($name, $desc) = @_;

	defined($name) || die "Category: adding without a name!";
	$name = substr($name, 0, 32);

	my $update;
	if (defined($desc)) {
		$update = &Woclema::DB::prepare(
				"INSERT INTO category (name, descr) " .
				"VALUES (&sa&, &sb&)");
		$update = &Woclema::DB::set($update,
				"a", $name,
				"b", $desc);
	} else {
		$update = &Woclema::DB::prepare(
				"INSERT INTO category (name) " .
				"VALUES (&sa&)");
		$update = &Woclema::DB::set($update, "a", $name);
	}
	&Woclema::DB::update($update);

	my $id = &Woclema::DB::get_sequence("category_id_seq");

	return $id;
}


=item add
Add a category.

0: name
(1: description)
r: category ID
=cut
sub add($$) {
	&Woclema::DB::transaction_start();
	my $id = &add_unsafe(@_);
	&Woclema::DB::transaction_end();
	return $id;
}


=item get_info
Get and write categoy info to cache.

0: id
=cut
sub get_info($) {
	my ($id) = @_;
	my ($query, $result);

	$query = &Woclema::DB::prepare(
			"SELECT name, descr FROM category " .
			"WHERE id = &da&");
	$query = &Woclema::DB::set($query, "a", $id);
	$result = &Woclema::DB::query($query);

	my $n = &Woclema::DB::row_count($result);

	if ($n > 1) {
		die "Category::get_info $id: $n results!"
	} elsif ($n == 1) {
		my ($name, $desc) = &Woclema::DB::get_row($result);
		$id_to_name[$id] = $name;
		$id_to_desc[$id] = $desc;
	}
}


=item get_name
Get category name (by id)

0: id
r: category name
=cut
sub get_name($) {
	my ($id) = @_;

	# Try from cache
	(defined($id_to_name[$id])) && return $id_to_name[$id];

	&get_info($id);

	return $id_to_name[$id];
}


=item get_desc
Get category name (by id)

0: id
r: category description
=cut
sub get_desc($) {
	my ($id) = @_;

	# Try from cache
	(defined($id_to_desc[$id])) && return $id_to_desc[$id];

	&get_info($id);

	return $id_to_desc[$id];
}


=item get_for_entry
Get categories for an entry.

0: entry id
r: (id, name) ...
=cut
sub get_for_entry($) {
	my ($id) = @_;
	my ($query, $result);

	$query = &Woclema::DB::prepare(
			"SELECT category.id, category.name " .
			"FROM category, entry_cat " .
			"WHERE entry_cat.entry = &da& " .
				"AND entry_cat.cat = category.id " .
			"ORDER BY category.name");
	$query = &Woclema::DB::set($query, "a", $id);

	$result = &Woclema::DB::query($query);

	my @ret;
	while (my ($c_id, $c_name) = &Woclema::DB::get_row($result)) {
		push(@ret, $c_id, $c_name);
	}

	return @ret;
}


=item get_by_name
Get category ID by category name.

0: name
r: ID or undef
=cut
sub get_by_name($) {
	my ($name) = @_;

	my ($query, $result);

	$query = &Woclema::DB::prepare("SELECT id FROM category " .
			"WHERE NAME ILIKE &sa&");
	$query = &Woclema::DB::set($query, "a", $name);
	$result = &Woclema::DB::query($query);

	if (&Woclema::DB::row_count($result) == 1) {
		my ($id) = &Woclema::DB::get_row($result);
		return $id;
	} elsif (&Woclema::DB::row_count($result) == 0) {
		return undef;
	} else {
		die "Category::get_by_str: More than one category " .
				"matching \"$name\"";
	}
}


=item get_list
Get a list of categories.

r: array of ("id \t name")
=cut
sub get_list() {
	my ($query, $result);

	$query = &Woclema::DB::prepare("SELECT id, name FROM category");
	$result = &Woclema::DB::query($query);

	my @ret;
	while (my ($id, $name) = &Woclema::DB::get_row($result)) {
		push(@ret, sprintf("%s\t%s", $id, $name));
	}

	return @ret;
}


=item remove_from_entry
Remove all categories from entry.
Does _not_ check if entry exists.

0: entry id
=cut
sub remove_from_entry($) {
	my ($id) = @_;
	my $update = &Woclema::DB::prepare("DELETE FROM entry_cat " .
			"WHERE entry = &da&");
	$update = &Woclema::DB::set($update, "a", $id);
	&Woclema::DB::update($update);
}


=item add_to_entry
Add given category to an entry.

0: entry ID
1: category ID
=cut
sub add_to_entry($$) {
	my ($entry, $cat) = @_;

	# See if entry exists
	my $exists = &Woclema::Entry::exists($entry);
	($exists == 1) || die "Category::add_to_entry: " .
			"No such entry: $entry";

	# See if category exists
	my $cat_name = &get_name($cat);
	(defined($cat_name)) || die "Category::add_to_entry: " .
			"No such category: $cat";

	my $update = &Woclema::DB::prepare("INSERT INTO entry_cat " .
			"(entry, cat) VALUES (&da&, &db&)");
	$update = &Woclema::DB::set($update,
			"a", $entry,
			"b", $cat);
	&Woclema::DB::update($update);
}


return 1;
