# Woclema - Weblog, Originally Command Line Editable/Manageable
# Copyright (C) 2006-2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package Woclema::DB;

use strict;
use warnings FATAL => 'all';
use diagnostics;
use Pg;

use Woclema::Config;


my $debug;

BEGIN {
	$debug = defined($ENV{"QUERY_STRING"}) ? 0 : 1;
	$debug |= 1 if (defined($ENV{"DEBUG"}));
}

$SIG{PIPE} = sub {
	print "Broken pipe!\n"; exit(0);
};


my $conn;


=item connect
Connect to database server (or die trying)
=cut
sub connect() {
	my ($addr, $port, $dbname, $user, $pass) = &Woclema::Config::get_db();

	# Order seems to matter!?
	my $cmd = "host=$addr";
	$cmd .= " port=$port"		if (defined($port));
	$cmd .= " dbname=$dbname";
	$cmd .= " user=$user"		if (defined($user));
	$cmd .= " password=$pass"	if (defined($pass));

	$conn = &Pg::connectdb($cmd);

	return ($conn->status == PGRES_CONNECTION_OK) ? 0 : -1;
}


=item check_connection
Check connection and try to (re-)establish it if necessary.
=cut
sub check_connection() {
	return if (defined($conn) && $conn->status == PGRES_CONNECTION_OK);

	for (my $i = 0; $i < 3; $i++) {
		my $r = &connect();
		if ($r == 0) {
			return;
		}
	}
	die $conn->errorMessage;
}


=item update
Execute database update

0: command
=cut
sub update() {
	my ($update) = @_;

	$update =~ /\x01[sdf][a-zA-Z]\x01/
			&& die "Statement unprepared!: $update";

	&check_connection();

	($debug == 1) && print "$update\n";
	my $result = $conn->exec($update);
	if ($result->resultStatus ne PGRES_COMMAND_OK) {
		printf STDERR "Woclema::DB::update failed: $update\n";
		die $conn->errorMessage;
	}
}


=item query
Execute database query

0: query
=cut
sub query($) {
	my ($query) = @_;

	$query =~ /\x01[sdf][a-zA-Z]\x01/
			&& die "Statement unprepared!: $query";

	&check_connection();

	($debug == 1) && print "$query\n";
	my $result = $conn->exec($query);
	if ($result->resultStatus ne PGRES_TUPLES_OK) {
		print STDERR "Woclema::DB::query failed: $query\n";
		die $conn->errorMessage;
	}

	return $result;
}


=item row_count
Get result row count

0: result
r: row count or -1 if result is undefined
=cut
sub row_count($) {
	my ($result) = @_;
	defined($result) || return -1;
	return $result->ntuples;
}


=item get_row
Get row from database result set

0: result
@: columns in queried order
=cut
sub get_row($) {
	my ($result) = @_;
	defined($result) || return undef;
	return $result->fetchrow;
}


=item get_sequence
Get sequence number

0: sequence name
r: n
=cut
sub get_sequence($) {
	my ($sequence) = @_;

	&check_connection();

	my $query = "SELECT currval('$sequence') AS id LIMIT 1";
	($debug == 1) && print "$query\n";
	my $result = &query($query);
	my ($seq) = &get_row($result);
	return $seq;
}


=item transaction_start
Start transaction block
=cut
sub transaction_start() {
	&update("BEGIN");
}


=item transaction_end
End transaction block
=cut
sub transaction_end() {
	&update("END");
}


=item bool_to_int
Convert database type boolean into integer (0 = true, 1 = false)

0: string
r: int
=cut
sub bool_to_int($) {
	my $b = shift;
	return $b eq "t" ? 1 : 0;
}


=item int_to_bool
Convert integer into database type boolean (0 = true, 1 = false)

0: int
r: string
=cut
sub int_to_bool($) {
	my $i = shift;
	return $i eq 1 ? "TRUE" : "FALSE";
}


=item numeric
Insert numeric value

0: value
1: 0 = value is int, 1 = value is float
=cut
sub prepare_numeric($$) {
	$_ = shift;
	my $float = defined($_[0]) && $_[0] == 1 ? 1 : 0;

	if ($float == 1) {
		s/[^0-9\-\.]//g;
	} else {
		s/[^0-9\-]//g;
	}

	if (/^-/) {
		s/-//g;
		s/^/-/;
	} else {
		s/-//g;
	}
	if ($float == 1) {
		s/\./+/;
		s/\.//g;
		s/\+/./g;
	} else {
		s/\.//g;
	}

	(/^-$/) && return "";

	return $_;
}


=item prepare
Prepare SQL statement

Also see set().

0: statement
r: statement
=cut
sub prepare($) {
	my ($string) = @_;
	$string =~ s/&(..)&/\x01$1\x01/g;
	# Check for duplicates
	my $t = "$string";
	my %ids;
	while (1) {
		my ($tag) = $t =~ /(\x01..\x01)/;
		last unless defined($tag);
		$t =~ s/\Q$tag\E//;
		my ($id) = $tag =~ /..(.)./;
		defined($ids{$id}) && die "Overlapping IDs: $string";
		$ids{$id} = 1;
	}
	return $string;
}


=item set
Set values in prepared SQL statement

IDs:
  d - integer
  f - float
  s - string
  b - boolean

0: prepared statement
1: id
2: value
r: statement
=cut
sub set($$$) {
	my $string = shift;
	while ($#_ + 1 >= 2) {
		my $id = shift;
		my $stuff = shift;

		my ($type) = $string =~ /\x01(.)$id\x01/;
		(defined($type)) || die "can't find id '$id' in '$string'!";

		if ($type eq "s") {
			$stuff =~ s/\\/\\\\/g;
			$stuff =~ s/'/''/g;
			$string =~ s/\x01s$id\x01/'$stuff'/;
		} elsif ($type eq "d") {
			$stuff = &prepare_numeric($stuff, 0);
			(length($stuff) > 0) or die "nothing to insert!";
			$string =~ s/\x01d$id\x01/$stuff/;
		} elsif ($type eq "f") {
			$stuff = &prepare_numeric($stuff, 1);
			(length($stuff) > 0) or die "nothing to insert!";
			$string =~ s/\x01f$id\x01/$stuff/;
		} elsif ($type eq "b") {
			$stuff = int_to_bool($stuff);
			$string =~ s/\x01b$id\x01/$stuff/;
		} else {
			die "unknown type!";
		}
	}
	return $string;
}


return 1;
