# Woclema - Weblog, Originally Command Line Editable/Manageable
# Copyright (C) 2007-2009 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package Woclema::Entry;

use strict;
use warnings FATAL => 'all';
use diagnostics;

use Woclema::DB;
use Woclema::HTML;


=item std_sql_for_timestamp
Return "standard" way to get timestamp info.

0: entry timestamp field
1: edit timestamp field
r: SQL
=cut
sub std_sql_for_timestamp($) {
	my ($entry, $edit) = @_;
	return "EXTRACT(YEAR FROM $entry AT TIME ZONE 'UTC') AS year, " .
		"EXTRACT(MONTH FROM $entry AT TIME ZONE 'UTC') AS month, " .
		"EXTRACT(DAY FROM $entry AT TIME ZONE 'UTC') AS day, " .
		"EXTRACT(HOUR FROM $entry AT TIME ZONE 'UTC') AS hour, " .
		"EXTRACT(MINUTE FROM $entry AT TIME ZONE 'UTC') AS minute, " .
		"EXTRACT(SECOND FROM $entry AT TIME ZONE 'UTC') AS second, " .
		"EXTRACT(EPOCH FROM $entry) AS epoch, " .
		"EXTRACT(EPOCH FROM $edit) AS edit_epoch";
}


=item std_sql_results
Parse "standard" entry query results.

0: SQL result
r: array of entries
=cut
sub std_sql_results($) {
	my ($result) = @_;
	my @ret;

	while (my ($id, $title, $text, $comments, $year, $month, $day,
				$hour, $minute, $second, $epoch, $edit) =
			&Woclema::DB::get_row($result)) {
		my %t = (
			"id"		=> $id,
			"title"		=> $title,
			"text"		=> $text,
			"comments"	=> $comments,
			"time_year"	=> $year,
			"time_month"	=> $month,
			"time_day"	=> $day,
			"time_hour"	=> $hour,
			"time_minute"	=> $minute,
			"time_second"	=> $second,
			"time_epoch"	=> $epoch,
			"edit_epoch"	=> $edit
			);
		push(@ret, \%t);
	}

	return @ret;
}


=item add
Add an entry.
Does not use transactions.

0: title
1: text
2: date (or undef)
3: public (define for "yes", defaults to "no")
r: entry ID
=cut
sub add_unsafe($$$$) {
	my ($title, $text, $date, $pub) = @_;

	defined($title) || die "Entry::add_unsafe: no title!";
	defined($text) || die "Entry::add_unsafe: no text!";
	$title = substr($title, 0, 128);

	# Insert entry
	my $update;
	$update = &Woclema::DB::prepare(
			"INSERT INTO entry (title, text, timest, public) " .
			"VALUES (&sa&, &sb&, &sc&, &bd&)");
	$update = &Woclema::DB::set($update,
			"a", $title,
			"b", $text,
			"c", defined($date) ? $date : "NOW()",
			"d", defined($pub) ? 1 : 0);
	&Woclema::DB::update($update);

	# Get ID
	my $id = &Woclema::DB::get_sequence("entry_id_seq");

	return $id;
}


=item add
Add an entry.

0: title
1: text
2: date (or undef)
3: public (define for "yes", defaults to "no")
r: entry ID
=cut
sub add($$$) {
	&Woclema::DB::transaction_start();
	my $id = &add(@_);
	&Woclema::DB::transaction_end();
	return $id;
}


=item edit
Edit entry.
Does _not_ check if entry ID exists.

0: id
1: title (or undef)
2: text (or undef)
=cut
sub edit($$) {
	my ($id, $title, $text) = @_;
	my $update;

	defined($id) || die "Entry::edit with no id";

	if (defined($title)) {
		$update = &Woclema::DB::prepare(
				"UPDATE entry SET title = &sa&, edit = NOW() " .
				"WHERE id = &db&");
		$update = &Woclema::DB::set($update,
				"a", $title,
				"b", $id);
		&Woclema::DB::update($update);
	}

	if (defined($text)) {
		$update = &Woclema::DB::prepare(
				"UPDATE entry SET text = &sa&, edit = NOW() " .
				"WHERE id = &db&");
		$update = &Woclema::DB::set($update,
				"a", $text,
				"b", $id);
		&Woclema::DB::update($update);
	}
}


=item exists
See if entry ID exists.

0: entry ID
1: must be published
r: 1 if exists
=cut
sub exists($) {
	my ($id, $pub) = @_;
	my ($query, $result);

	if (defined($pub)) {
		$query = &Woclema::DB::prepare(
			"SELECT id FROM entry " .
			"WHERE id = &da& AND public = &bb&");
		$query = &Woclema::DB::set($query,
				"a", $id,
				"b", 1);
	} else {
		$query = &Woclema::DB::prepare(
			"SELECT id FROM entry WHERE id = &da&");
		$query = &Woclema::DB::set($query, "a", $id);
	}
	$result = &Woclema::DB::query($query);
	return &Woclema::DB::row_count($result) == 1 ? 1 : 0;
}


=item set_public
Set/unset entry public.

0: id
1: public (0 == false)
=cut
sub set_public($$) {
	my ($id, $pub) = @_;

	my $update = &Woclema::DB::prepare(
			"UPDATE entry SET public = &ba& WHERE id = &db&");
	$update = &Woclema::DB::set($update,
			"a", $pub,
			"b", $id);
	&Woclema::DB::update($update);
}


=item get
Get entry.

0: ID
1: 1 = accept unpublished
r: entry
=cut
sub get($) {
	my ($id, $accept_unpub) = @_;
	my ($query, $result);
	$accept_unpub = 0 unless defined($accept_unpub);

	if ($accept_unpub == 0) {
		$query = &Woclema::DB::prepare(
				"SELECT title, text, comments, " .
				&std_sql_for_timestamp("timest", "edit") . " " .
				"FROM entry " .
				"WHERE id = &da& AND public = &bb&");
		$query = &Woclema::DB::set($query,
				"a", $id,
				"b", 1);
	} else {
		$query = &Woclema::DB::prepare(
				"SELECT title, text, comments, " .
				&std_sql_for_timestamp("timest", "edit") . " " .
				"FROM entry " .
				"WHERE id = &da&");
		$query = &Woclema::DB::set($query, "a", $id);
	}

	$result = &Woclema::DB::query($query);

	my %ret;

	return %ret if (&Woclema::DB::row_count($result) != 1);

	my ($title, $text, $comments, $year, $month, $day,
			$hour, $minute, $second, $epoch) =
		&Woclema::DB::get_row($result);
	%ret = (
		"id"		=> $id,
		"title"		=> $title,
		"text"		=> $text,
		"comments"	=> $comments,
		"time_year"	=> $year,
		"time_month"	=> $month,
		"time_day"	=> $day,
		"time_hour"	=> $hour,
		"time_minute"	=> $minute,
		"time_second"	=> $second,
		"time_epoch"	=> $epoch
		);

	return %ret;
}


=item comment_counter_unsafe
Adjust comment counter by given value.
Does not use transaction.

0: entry id
1: adjustment
=cut
sub comment_counter_unsafe($$) {
	my ($id, $adj) = @_;

	# Get old value
	my ($query, $result);
	$query = &Woclema::DB::prepare(
			"SELECT comments FROM entry WHERE id = &da&");
	$query = &Woclema::DB::set($query, "a", $id);

	$result = &Woclema::DB::query($query);
	(&Woclema::DB::row_count($result) == 1)
			|| die "Entry::comment_counter_unsafe: " .
				"bad query results for $id";

	my ($count) = &Woclema::DB::get_row($result);

	# Update value
	my $update = &Woclema::DB::prepare(
			"UPDATE entry SET comments = &da& WHERE id = &db&");
	$update = &Woclema::DB::set($update,
			"a", $count + 1,
			"b", $id);
	&Woclema::DB::update($update);
}


=item comment_counter
Adjust comment counter by given value.

0: entry id
1: adjustment
=cut
sub comment_counter($$) {
	&Woclema::DB::transaction_start();
	&comment_counter_unsafe(@_);
	&Woclema::DB::transaction_end();
}


=item get_list_before_time
Get list of entries. The latest first.

0: latest ID (must be defined)
1: n, defaults to 5
2: need to be published, defaults to 1 == yes
r: array of entries
=cut
sub get_list_before_time($$$) {
	my ($latest, $n, $published) = @_;
	my ($query, $result);

	$n = 5 unless defined($n);
	$published = 1 unless defined($published);

	$query = &Woclema::DB::prepare(
			"SELECT id, title, text, comments, " .
			&std_sql_for_timestamp("timest", "edit") . " " .
			"FROM entry_with_epoch " .
			"WHERE epoch < &da& AND public = &bb& " .
				"AND id != -1 " .
			"ORDER BY epoch DESC " .
			"LIMIT &dc&");
	$query = &Woclema::DB::set($query,
			"a", $latest,
			"b", $published,
			"c", $n);

	$result = &Woclema::DB::query($query);
	return &std_sql_results($result);
}


=item get_list_between_time
Get list of entries. The latest first.

0: start date
1: end date
r: array of entries
=cut
sub get_list_between_time($$) {
	my ($first, $last) = @_;
	my ($query, $result);

	$query = &Woclema::DB::prepare(
			"SELECT id, title, text, comments, " .
			&std_sql_for_timestamp("timest", "edit") . " " .
			"FROM entry_with_epoch " .
			"WHERE public = &ba& AND " .
				"epoch >= &db& AND epoch < &dc& " .
			"ORDER BY epoch DESC");
	$query = &Woclema::DB::set($query,
			"a", 1,
			"b", $first,
			"c", $last);

	my @ret;
	$result = &Woclema::DB::query($query);
	return &std_sql_results($result);
}


=item exist_range
See if published weblog entries exist in given range.

0: start date (UNIX epoch)
1: end date (UNIX epoch)
r: list of dates (UNIX epoch)
=cut
sub exist_range($$) {
	my ($start, $end) = @_;

	my ($query, $result);
	
	$query = &Woclema::DB::prepare(
			"SELECT EXTRACT(EPOCH FROM timest) AS epoch " .
			"FROM entry " .
			"WHERE public = &ba& " .
			"AND timest >= TIMESTAMP WITH TIME ZONE 'epoch' " .
				"+ &db& * INTERVAL '1 second' " .
			"AND timest < TIMESTAMP WITH TIME ZONE 'epoch' " .
				"+ &dc& * INTERVAL '1 second' " .
			"ORDER BY epoch");
	$query = &Woclema::DB::set($query,
			"a", 1,
			"b", int($start),
			"c", int($end));
	$result = &Woclema::DB::query($query);

	my @ret;
	while (my ($date) = &Woclema::DB::get_row($result)) {
		push(@ret, $date);
	}

	return @ret;
}


=item get_for_cat
Get entries for given category.

0: category ID
r: list of entries
=cut
sub get_for_cat($) {
	my ($cat) = @_;
	my ($query, $result);
	
	$query = &Woclema::DB::prepare(
			"SELECT id, title, text, comments, " .
			&std_sql_for_timestamp("timest", "edit") . " " .
			"FROM entry_with_epoch, entry_cat " .
			"WHERE entry_cat.entry = id AND entry_cat.cat = &da& " .
				"AND public = &bb& " .
			"ORDER BY epoch DESC");
	$query = &Woclema::DB::set($query,
			"a", $cat,
			"b", 1);

	my @ret;
	$result = &Woclema::DB::query($query);
	return &std_sql_results($result);

	return @ret;
}


return 1;
