# Woclema - Weblog, Originally Command Line Editable/Manageable
# Copyright (C) 2006-2009 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package Woclema::HTML;

use strict;
use warnings FATAL => 'all';
use diagnostics;

use Woclema::Config;
use Woclema::Common;

my @content;
my @sidebar;
my $robot;

=item print
Print page.
=cut
sub print() {
	open(FILE, "< template") or die "Cannot read template file: $!";
	my $t = join("\n", <FILE>);
	close(FILE);

	$t =~ s/<\/head>/\t<meta name="robots" content="$robot" \/>\n<\/head>/i
		if (defined($robot));

	my $x;
	
	$x = ($#content >= 0) ? join("\n", @content) : "";
	$t =~ s/__WOCLEMA_ENTRY__/$x/;

	$x = ($#sidebar >= 0) ? join("\n", @sidebar) : "";
	$t =~ s/__WOCLEMA_SIDEBAR__/$x/;

	$x = sprintf("%.3f", &Woclema::Common::process_time());
	$t =~ s/__WOCLEMA_TIME__/$x/;

	print "$t\n";
}


=item set_robot
Set "robots" meta tag

0: value
=cut
sub set_robot($) {
	($robot) = @_;
}


=item add_content
Write page content.

@: content (lines)
=cut
sub add_content(@) {
	push(@content, @_);
}


=item add_sidebar
Write content to sidebar.

@: content (lines)
=cut
sub add_sidebar(@) {
	push(@sidebar, @_);
}


=item bad_param
Act upon bad parameters

0: msg
=cut
sub bad_param($) {		# XXX
	my ($msg) = @_;
	print "<h1>LOL WTF n00b I pwn</h1>\n";
	(defined($msg)) && print "<p>$msg</p>\n";
	&print_footer();
	exit(0);
}


=item esc_param
Make parameters safe

@: options
r: parameter string
=cut
sub esc_param(@) {		# XXX
	my $url = "";
	my $first = 1;
	foreach (@_) {
		my $i = "$_";
		$i =~ s/#/%23/;
		$i =~ s/&/%26/;
		$i =~ s/\?/%3f/;
		if ($first == 0) {
			$url .= "&amp;";
		} else {
			$first = 0;
		}
		$url .= $i;
	}
	return $url;
}


=item truncate_url
Truncate URL nicely

0: url
1: len = 60
2: end = 10
r: url
=cut
sub truncate_url($$$) {		# XXX
	my ($url, $len, $end) = @_;
	$len = 60 if (! defined($len));
	$end = 10 if (! defined($end));

	my $rl = length($url);
	($rl <= $len) && return $url;

	# Can we do it nicely and cleanly?
	my ($domain, $path) = $url =~ /([a-z]+:\/\/.*?)\/(.*)/;
	if (! defined($domain)) {
		$domain = $url;
		$path = "";
	}
	if (length($domain) < $len) {
		my @t = split(/\//, $path);
		my $count = $#t + 1;
		while ($count > 1) {
			my $newurl = $domain . "/...";
			for (my $i = $#t + 1 - $count; $i < $#t + 1; $i++) {
				$newurl .= "/" . $t[$i];
			}
			if (length($newurl) <= $len) {
				return $newurl;
			}
			$count--;
		}
	}

	# Fallback...
	return (substr($url, 0, $len - $end - 3) . "..." .
			substr($url, $rl - $end));
}


=item htmlize
Make text proper HTML

0: string
r: html
=cut
sub htmlize($) {
	my ($t) = @_;
	$t =~ s/&/\&amp;/g;
	$t =~ s/</\&lt;/g;
	$t =~ s/>/\&gt;/g;
	return $t;
}



=item shorten
Shorten HTML blob to N words

0: string
1: number of words or -1 for full text (after checking)
r: HTML or undef
=cut
sub shorten($$) {
	my ($html, $n) = @_;
	($n == 0) && return undef;

	my $debug = 0;
	my @tags;
	my $ret = "";
	my $whitespace = "";
	my $truncated = 0;

	foreach my $chunk (split(/</, $html)) {
		($debug) && print "chunk: -$chunk-\n";
		my ($tag, $text) = split(/>/, $chunk, 2);
		if (! defined($text)) {
			$text = $chunk;
			undef $tag;
		}
		($debug) && print " text: $text\n";

		if (defined($tag)) {
			my $otag = $tag;
			($debug) && print "  tag: -$tag-\n";
			if ($tag =~ /^\//) {
				$tag =~ s/^\///;
				$tag =~ s/\s.*//s;
				if ($#tags == -1 || $tags[$#tags] ne $tag) {
					($debug && $#tags == -1) &&
						print "empty stack\n";
					($debug && $tags[$#tags] ne $tag) &&
						print "in stack: $tags[$#tags]\n";
					return undef;
				}
				pop(@tags);
				$ret .= "$whitespace<$otag>";
				$whitespace = "";
				($debug) && print "c tag: $tag\n";
			} else {
				$tag =~ s/\s.*//s;
				$ret .= "$whitespace<$otag>";
				$whitespace = "";
				if ($otag =~ /\/\s*$/) {
					($debug) && print "i tag: $tag\n";
				} else {
					push(@tags, $tag);
					($debug) && print "o tag: $tag\n";
				}
			}
		}

		(defined($text)) || last;

		# $n != 0 -- that'll take care of ($n > 0 || $n == -1)
		while (length($text) > 0 && $n != 0) {
			my $otext = $text;
			my ($word, $new_ws);
			($word, $new_ws, $text) = $text =~ /(.*?)(\s+)(.*)/s;
			if (! defined($text)) {
				$word = $otext;
				$new_ws = "";
				$text = "";
			}
#			($debug) && print " word: $word\n";
#			($debug) && print "   ws: ->$new_ws<-\n";
#			($debug) && print " text: $text\n";

			$n--;
			$ret .= $whitespace . $word;
			$whitespace = $new_ws;
		}
		if ($n == 0) {
			$truncated = 1;
			last;
		}
	}
	while ($#tags >= 0) {
		$ret .= "</" . pop(@tags) . ">";
	}
	if ($truncated) {
		$ret .= "\n<p>...</p>";
	}
	return $ret;
}



return 1;
