#! /usr/bin/perl
#
# Woclema - Weblog, Originally Command Line Editable/Manageable
# Copyright (C) 2006-2009 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

use Woclema::Entry;
use Woclema::Category;
use Woclema::Link;
use Woclema::DB;
use Woclema::Config;
use Woclema::HTML;


=item get_line
Get line from STDIN.

r: line without linefeed
=cut
sub get_line() {
	$_ = <STDIN>;
	chomp;
	return $_;
}


=item read_file
Read blog entry file.

0: filename
r: info
=cut
sub read_file($) {
	my ($file) = @_;
	open(FILE, "< $file") or die "Cannot read $file.";

	my %info;
	my @cats;
	my $phase = 0;

	while (<FILE>) {
		if ($phase == 0) {
			chomp;
			if (/^Title: /) {
				s/^Title: //;
				$info{'title'} = $_;
			} elsif (/^Categories: /) {
				s/^Categories: //;
				push(@cats, split(/,\s+/, $_));
			} elsif (/^Date: /) {
				s/^Date: //;
				$info{'date'} = $_;
			} elsif (! /./) {
				$phase = 1;
			}
		} else {
			$info{'text'} .= "$_";
		}
	}
	close(FILE);
	
	$info{'cats'} = join("\0", @cats);

	return %info;
}


sub main() {
	my $file = $ARGV[0];
	my $id = $ARGV[1];

	my %entry = &read_file($file);
	defined($entry{'title'}) || die "No title for \"$file\"";
	defined($entry{'text'}) || die "No text for \"$file\"";

	if (! defined(&Woclema::HTML::shorten($entry{'text'}, -1))) {
		print "Check your HTML.\n";
		exit(1);
	}

#	print "text: $entry{'text'}\n";

	&Woclema::DB::transaction_start();

	if (! defined($id)) {
		print "No entry ID given. Adding NEW weblog entry.\n";
		print "\n";
		print "Title: $entry{'title'}\n";
		if (defined($entry{'cats'})) {
			print "Cats:";
			print " $_," foreach (split(/\0/, $entry{'cats'}));
			print "\n";
		}
		print "Date: $entry{'date'}\n" if (defined($entry{'date'}));
		print "\n";
		print "Continue [y] / Cancel [n]?\n";
		$_ = &get_line();
		exit(1) unless ($_ eq "y");

		print "Adding entry...\n";
		$id = &Woclema::Entry::add_unsafe(
				$entry{'title'}, $entry{'text'},
				$entry{'date'});
		print "New ID: $id\n";
	} else {
		my %old_entry = &Woclema::Entry::get($id, 1);
		if (! defined($old_entry{'id'})) {
			print "Entry $id does not exist!\n";
			exit(1);
		}
		print "Editing old entry...\n";
		if ($old_entry{'title'} ne $entry{'title'}) {
			print "\n";
			print "Old title: $old_entry{'title'}\n";
			print "New title: $entry{'title'}\n";
			print "\n";
			print "Continue [y] / Cancel [n]?\n";
			$_ = &get_line();
			exit(1) unless ($_ eq "y");
		}
		&Woclema::Entry::edit($id, $entry{'title'}, $entry{'text'});
	}

	print "Clearing old categories...\n";
	&Woclema::Category::remove_from_entry($id);

	print "Setting categories...\n";
	foreach my $cat_str (split(/\0/, $entry{'cats'})) {
		my $cat_id = &Woclema::Category::get_by_name($cat_str);
		if (! defined($cat_id)) {
			print "Category \"$cat_str\" not defined.\n";
			print "Create [y] / skip [n] / cancel [c]?\n";
			$_ = &get_line();
			if ($_ eq "y") {
				print "Adding new category...\n";
				$cat_id = &Woclema::Category::add_unsafe(
						$cat_str);
			} elsif ($_ eq "n") {
				next;
			} elsif ($_ eq "c") {
				exit(1);
			}
		}

		defined($cat_id) || die "Category ID still undefined!";

		print "Attaching to category...\n";
		&Woclema::Category::add_to_entry($id, $cat_id);
	}

	&Woclema::DB::transaction_end();

	$Woclema::Config::rewrite = 0;
	my $preview_url = &Woclema::Link::entry($id) . "&unpub=1";
	$preview_url =~ s/\/\/+/\//g;

	print <<EOF;
	
Done with entry #$id

Preview: $preview_url
Edit:    ./write_entry.pl $file $id 
Publish: ./publish.pl $id

EOF
}


&main();
exit(0);
